
;-------------------------------------------------------------------------------------------------------
; Copyright (c) 2011-2012, Ilya N.  All rights reserved.
;
; This file may be distributed and/or modified under the terms of the GNU GPLv2 as published by the
; Free Software Foundation and appearing in the file gpl-2.0.txt included in the packaging of this file.
; Copy of the License Agreement can also be found at www.gnu.org
;
; THIS SOFTWARE IS PROVIDED BY Ilya N. ''AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT
; NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
; DISCLAIMED. IN NO EVENT SHALL Ilya N. BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
; EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
; GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
; THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
; ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
;
; This copyright notice MUST APPEAR in all copies of the script!
;-------------------------------------------------------------------------------------------------------


parse_EDID:
	push	ds es

	xor	ecx, ecx
	mov	ds, cx
	mov	es, cx
	mov	bx, 1
	mov	edi, edid_buff
	xor	edx, edx
	mov	eax, $4f15
	int	$10
	cmp	ax, $4f
	jnz	edid_est_std_done	  ; EDID not supported

	xor	edx, edx
	xor	ecx, ecx
	mov	ebx, edid_buff
	mov	ebp, 64
	; cheksum: 128 bytes must sum up to 0
.checksum:
	add	dl, [bx]
	add	cl, [bx+1]
	add	bx, 2
	dec	bp
	jnz	.checksum
	add	dl, cl
	jnz	edid_est_std_done	  ; EDID not supported

;---------------------------------------------------------------------------------------------------

	mov	dx, [edid_buff+$12]	  ; +0x12 version
	mov	bp, [edid_buff+$15]	  ; +0x15 horiz size in centimeters, +0x16 vert size //  or AR
	mov	cx, [edid_buff+$23]	  ; +0x23 established timming 1 & 2

	mov	[cs:edid_timings], cx
	mov	[cs:edid_version], dx	  ; edid version(lowest byte) and revision(highest byte)
	mov	[cs:edid_sizes], bp	  ; width (lowest byte) and height  //  OR aspect ratio
	mov	bx, [cs:edid_1stEntry]

	xor	dx, dx
	test	cx, 1'00000000b 	  ; 1280x1024
	jz	@f
	inc	[cs:edid_count]
	mov	dword [bx], $400'0500	  ; resolution
	mov	word [bx+4], 0		  ; 8bit refresh rate(for future),   =$ff if rate is >= 255
	add	bx, 6			  ;                                  =0 if rate is unknown
@@:
	test	cx, 11110'00000000b	  ; 1024x768
	jz	@f
	inc	[cs:edid_count]
	mov	dword [bx], $300'0400
	mov	word [bx+4], dx
	add	bx, 6
@@:
	test	cx, 11000000'00000011b	  ; 800x600
	jz	@f
	inc	[cs:edid_count]
	mov	dword [bx], $258'0320
	mov	word [bx+4], dx
	add	bx, 6
@@:
	test	cx, 00111100b		  ; 640x480
	jz	@f
	inc	[cs:edid_count]
	mov	dword [bx], $1e0'0280
	mov	word [bx+4], dx
	add	bx, 6
@@:

;---------------------------------------------------------------------------------------------------
	mov	di, edid_buff+$26
edid_std_timing:
	movzx	ecx, word [di]
	add	di, 2

	test	cl, cl
	jz	.next
	cmp	cx, $2020
	jz	.next
	cmp	cx, $0101
	jz	.next

	mov	dl, ch
	and	ch, 00111111b		  ; isolate refresh rate -> ch
	add	ch, 60
	mov	[bx+4], ch		  ; save refresh rate

	shr	dl, 6			  ; isolate aspect ratio -> dl
	jz	.1_1
	cmp	dl, 2
	jb	.4_3
	jz	.5_4
.16_9:
	mov	ebp, $9'0010		  ; ebp[15:0] - divider,  ebp[31:16] - multiplier
	jmp	@f
.5_4:
	mov	ebp, $4'0005
	jmp	@f
.4_3:
	mov	ebp, $3'0004
	jmp	@f
.1_1:
	mov	ebp, $a'0010
	cmp	byte [cs:edid_version], 1     ; check version, if above 1.2 then 16:10 ratio
	ja	@f
	cmp	byte [cs:edid_version+1], 2
	ja	@f
	mov	ebp, $1'0001
@@:
	movzx	eax, cl
	lea	eax, [eax*8+248]
	mov	[bx], ax		; save width
@@:
	xor	edx, edx
	div	bp
	shr	ebp, 16
	imul	eax, ebp
	inc	byte [cs:edid_count]
	mov	[bx+2], ax		    ; save height
	add	bx, 6

.next:
	cmp	di, edid_buff+$36
	jb	edid_std_timing

;===================================================================================================
;   Monitor Aspect Ration(AR)     ||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
;===================================================================================================
    ; 16:9 (79) 1.777         16:10 (61) 1.600         4:3 (34) 1.333          5:4 (26) 1.250

	movzx	ecx, word [cs:edid_sizes]
	mov	edx, $ffff
	test	cx, cx
	jz	.monitor_AR_done	 ; undefined aspect ratio and/or dimensions
	test	cl, cl
	jz	.monitor_AR_done	 ; ignore Portrait mode
	test	ch, ch
	jnz	.width_height

	and	ecx, $ff
	cmp	cl, 79			 ; our own code rounds down so we adjust (79->78)
	jnz	@f			 ;   Other values won't matter because "standart" and
	dec	cl			 ;   "established" timings are limited to certain aspect ratios
@@:
	lea	edx, [ecx+99]
	jmp	.monitor_AR_done

.width_height:
	; width & height are heavily rounded so we need to aproximate ranges resulted
	; after width and height division:

	movzx	eax, cl 		 ; al = width
	shr	cx, 8			 ; cl = height
	xor	edx, edx
	imul	eax, eax, 1000
	div	ecx

	mov	edx, 177
	cmp	eax, 1688
	jae	.monitor_AR_done	 ; 16:9

	mov	dl, 160
	cmp	ax, 1467
	jae	.monitor_AR_done	 ; 16:10

	mov	dl, 133
	cmp	ax, 1292
	jae	.monitor_AR_done	 ; 4:3

	mov	dl, 125
	cmp	ax, 1000
	ja	.monitor_AR_done	 ; 5:4

	mov	dl, 100
	jz	.monitor_AR_done	 ; 1:1

	mov	dx, $ffff

.monitor_AR_done:
	mov	[cs:monitorAR], dx

edid_est_std_done:
	pop	es ds
	ret


;===================================================================================================
;     pickVbeMode      |||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
;===================================================================================================

	; pick "best" videomode supported by all hardware
	;------------------------------------------------------------------
	; 1) prefer width & height according to monitor Aspect Ratio(AR)
	; 2) go for largest width
	; 3) go for highest bits per pixel

pickVbeMode:
	push	ds
	xor	ax, ax
	mov	ds, ax

	movzx	di, [cs:vbe_count]
	xor	ebp, ebp		 ; bp - sel width, ebp[31]=1 if valid mode for specific AR exists
	xor	ebx, ebx		 ; bl - selected bps, ebx[23:8] - selected vbe entry address

	mov	si, modes_table+size_of_step	 ; "+size_of_step" skips VGA entry which must be 1st
	sub	di, 1			 ; account for VGA entry
	jbe	.exit
	cmp	[cs:edid_count], 0
	jz	.exit

.read_VBE_mode:
	mov	ecx, [si]

	; loop thru EDID entries
	mov	dx, si			 ; preserve SI
	movzx	ax, [cs:edid_count]
	mov	si, [cs:edid_1stEntry]
.edidEntry:
	cmp	[si], ecx
	jz	@f			 ; jump if found matching width/height
	add	si, 6
	dec	ax
	jnz	.edidEntry
	mov	si, dx
	jmp	.next_vbe_mode
@@:	mov	si, dx			; restore SI


	movzx	eax, cx 		; eax = width
	shr	ecx, 16 		; ecx = height
	jz	.next_vbe_mode
	imul	eax, eax, 100		; eax = width * 100
	xor	edx, edx
	div	ecx			; ax = aspect ratio


	cmp	[cs:monitorAR], ax
	jnz	.entry_no_AR
	cmp	eax, $fffe
	ja	.entry_no_AR

	bts	ebp, 31 		; bit31=1 - indicates that an entry with correct AR exists
	jc	.AR_done		; proceed if this is not 1st AR entry
	xor	ebx, ebx		; flush info about selected entry in case
	mov	ebp, $8000'0000 	;    we went thru entries with incorrect AR
	jmp	.AR_done

.entry_no_AR:
	bt	ebp, 31 		; go to next entry if aspect ratio of current entry doesn't match
	jc	.next_vbe_mode		;                     while aspect ratio of some other entry does
.AR_done:

	mov	dx, [si]
	cmp	bp, dx
	ja	.next_vbe_mode
	mov	bp, dx			; bp = max width
	jb	.save_bps		; auto-save BPS if current width > previous

	cmp	[si+8], bl
	jb	@f
.save_bps:
	mov	bl, [si+8]		; bl = max bits per pixel
@@:
	ror	ebx, 8
	mov	bx, si			; save offset
	rol	ebx, 8

.next_vbe_mode:
	add	si, size_of_step
	dec	di
	jnz	.read_VBE_mode

	shr	ebx, 8
	sub	bx, modes_table
	jc	.exit			; just in case
	mov	[cs:vbe_selectedMode], bx
.exit:
	pop	ds
ret





